/*
 * SPDX-FileCopyrightText: 2023 Helge
 *
 * SPDX-License-Identifier: MIT
 */

body {
    background-color: darkcyan;
    font-family: serif;
}

@media (min-device-width: 800px) {
    main {
        margin-top: 50px;
        max-width: 800px;
        margin-left: calc(50% - 400px);
        box-shadow: 5px 5px black;
    }
}

h1,
h2,
h3 {
    text-align: center;
}

main {
    background-color: white;
    width: 100%;
    border: 1px solid black;
    padding: 20px;
    border-radius: 20px;
}

input[type=text] {
    margin: 15px 30px;
    padding: 10px;
    width: calc(100% - 80px);
}

input[type=checkbox] {
    width: 24px;
    height: 24px;
    display: inline-block;
    vertical-align: middle;
    background-size: 100%;
    margin: 10px;
}

input[type=checkbox]+label {
    display: inline-block;
    vertical-align: middle;
}

details li {
    list-style: none;
}


input[type=submit] {
    background-color: darkcyan;
    border: none;
    color: #fff;
    padding: 15px 30px;
    text-decoration: none;
    display: block;
    margin: auto;
    margin-bottom: 20px;
    cursor: pointer;
}

pre {
    white-space: pre-wrap;
}


table {
    border-collapse: collapse;
    width: 100%;
}

td, th {
    border: 1px solid #ddd;
    padding: 8px;
    vertical-align: top;
}

tr:nth-child(even) {
    background-color: #f2f2f2;
}

tr:hover {
    background-color: #ddd;
}

th {
    padding-top: 12px;
    padding-bottom: 12px;
    text-align: left;
    background-color: #04AA6D;
    color: white;
}

tr.warning {
    background-color: rgba(255, 0, 0, 0.2)
}

tr.warning:nth-child(even) {
    background-color: rgba(255, 0, 0, 0.4)
}

code {
    line-height: 125%;
    font-size: 125%;
}

ul.remarks {
    margin: 0;
    padding: 5px;
    list-style: " -";
    font-size: 80%;
}